@@ -11,12 +11,11 @@
 #include "geo/polylinedecoder_p.h"
 
 #include <KPublicTransport/Journey>
-#include <KPublicTransport/Line>
 #include <KPublicTransport/Platform>
+#include <KPublicTransport/RentalVehicle>
 #include <KPublicTransport/Stopover>
 #include <KPublicTransport/Vehicle>
 
-#include <QColor>
 #include <QDateTime>
 #include <QDebug>
 #include <QJsonArray>
@@ -245,9 +244,20 @@ std::vector<Line> HafasMgateParser::parseLines(const QJsonArray &prodL, const st
     lines.reserve(prodL.size());
     for (const auto &prodV : prodL) {
         const auto prodObj = prodV.toObject();
+        const auto prodCls = prodObj.value(QLatin1String("cls")).toInt();
+
         Line line;
-        line.setName(prodObj.value(QLatin1String("name")).toString());
-        line.setMode(parseLineMode(prodObj.value(QLatin1String("cls")).toInt()));
+        line.setMode(parseLineMode(prodCls));
+
+        if (std::binary_search(m_lineNumberProducts.begin(), m_lineNumberProducts.end(), prodCls)) {
+            line.setName(prodObj.value(QLatin1String("line")).toString());
+            if (line.name().isEmpty()) {
+                line.setName(prodObj.value(QLatin1String("nameS")).toString());
+            }
+        }
+        if (line.name().isEmpty()) {
+            line.setName(prodObj.value(QLatin1String("name")).toString());
+        }
 
         const auto icoIdx = prodObj.value(QLatin1String("icoX")).toInt();
         if ((unsigned int)icoIdx < icos.size()) {
@@ -530,10 +540,58 @@ static Path parsePolyG(const QJsonObject &obj, const std::vector<Path> &paths)
         return {};
     }
     const auto polyX = polyXL.at(0).toInt();
-    if (polyX >= 0 && polyX < (int)paths.size()) {
-        return paths[polyX];
+    if (polyX < 0 || polyX >= (int)paths.size()) {
+        return {};
     }
-    return {};
+
+    const auto segL = obj.value(QLatin1String("segL")).toArray();
+    auto path = paths[polyX];
+    if (segL.isEmpty() || path.sections().size() != 1) {
+        return path;
+    }
+
+    const auto poly = path.sections()[0].path();
+    std::vector<PathSection> pathSections;
+    pathSections.reserve(segL.size());
+    for (const auto &segV : segL) {
+        const auto segObj = segV.toObject();
+        PathSection sec;
+        sec.setDescription(segObj.value(QLatin1String("manTx")).toString());
+        const auto polyS = segObj.value(QLatin1String("polyS")).toInt();
+        const auto polyE = segObj.value(QLatin1String("polyE")).toInt();
+        if (polyS >= 0 && polyS < poly.size() && polyE >= polyS && polyE < poly.size()) {
+            QPolygonF subPoly;
+            subPoly.reserve(polyE - polyS + 1);
+            std::copy(poly.begin() + polyS, poly.begin() + polyE + 1, std::back_inserter(subPoly));
+            sec.setPath(std::move(subPoly));
+        }
+        pathSections.push_back(std::move(sec));
+    }
+    path.setSections(std::move(pathSections));
+    return path;
+}
+
+static void parseMcpData(const QJsonObject &obj, Location &loc)
+{
+    const auto mcp = obj.value(QLatin1String("mcp")).toObject();
+    if (mcp.isEmpty()) {
+        return;
+    }
+    const auto mcpData = mcp.value(QLatin1String("mcpData")).toObject();
+    // TODO mcpData.provider to vehicle type lookup from meta data
+    const auto providerName = mcpData.value(QLatin1String("providerName")).toString();
+    qDebug() << providerName << mcpData;
+    if (providerName.isEmpty()) {
+        return;
+    }
+
+    // ### are we even sure this is always a station? how does this distinguish free floating vehicles?
+    RentalVehicleNetwork network;
+    network.setName(providerName);
+    RentalVehicleStation station;
+    station.setNetwork(network);
+    loc.setData(station);
+    loc.setType(Location::RentedVehicleStation);
 }
 
 std::vector<Journey> HafasMgateParser::parseTripSearch(const QJsonObject &obj)
@@ -572,7 +630,9 @@ std::vector<Journey> HafasMgateParser::parseTripSearch(const QJsonObject &obj)
             section.setExpectedDepartureTime(parseDateTime(dateStr, dep.value(QLatin1String("dTimeR")), dep.value(QLatin1String("dTZOffset"))));
             const auto fromIdx = dep.value(QLatin1String("locX")).toInt(-1);
             if ((unsigned int)fromIdx < locs.size()) {
-                section.setFrom(locs[fromIdx]);
+                auto loc = locs[fromIdx];
+                parseMcpData(dep, loc);
+                section.setFrom(std::move(loc));
             }
             section.setScheduledDeparturePlatform(dep.value(QLatin1String("dPlatfS")).toString());
             section.setExpectedDeparturePlatform(dep.value(QLatin1String("dPlatfR")).toString());
@@ -585,7 +645,9 @@ std::vector<Journey> HafasMgateParser::parseTripSearch(const QJsonObject &obj)
             section.setExpectedArrivalTime(parseDateTime(dateStr, arr.value(QLatin1String("aTimeR")), arr.value(QLatin1String("aTZOffset"))));
             const auto toIdx = arr.value(QLatin1String("locX")).toInt(-1);
             if ((unsigned int)toIdx < locs.size()) {
-                section.setTo(locs[toIdx]);
+                auto loc = locs[toIdx];
+                parseMcpData(arr, loc);
+                section.setTo(loc);
             }
             section.setScheduledArrivalPlatform(arr.value(QLatin1String("aPlatfS")).toString());
             section.setExpectedArrivalPlatform(arr.value(QLatin1String("aPlatfR")).toString());
@@ -673,11 +735,38 @@ std::vector<Journey> HafasMgateParser::parseTripSearch(const QJsonObject &obj)
                 });
                 parseTrainComposition(dep, section, loadInfos, platforms, vehicles);
                 section.setPath(parsePolyG(jnyObj, paths));
-            } else if (typeStr == QLatin1String("WALK") || typeStr == QLatin1String("TRSF")) {
+            } else {
                 const auto gis = secObj.value(QLatin1String("gis")).toObject();
                 section.setDistance(gis.value(QLatin1String("dist")).toInt());
                 section.setPath(parsePolyG(gis, paths));
-                section.setMode(typeStr == QLatin1String("WALK") ? JourneySection::Walking : JourneySection::Transfer);
+                if (typeStr == QLatin1String("WALK")) {
+                    section.setMode(JourneySection::Walking);
+                } else if (typeStr == QLatin1String("TRSF")) {
+                    section.setMode(JourneySection::Transfer);
+                } else if (typeStr == QLatin1String("BIKE")) {
+                    if (section.from().type() == Location::RentedVehicleStation) {
+                        section.setMode(JourneySection::RentedVehicle);
+                        RentalVehicle v;
+                        v.setNetwork(section.from().rentalVehicleStation().network());
+                        v.setType(RentalVehicle::Bicycle); // TODO we also get here for kick scooters?
+                        section.setRentalVehicle(v);
+                    } else {
+                        section.setMode(JourneySection::IndividualTransport);
+                        section.setIndividualTransport({ IndividualTransport::Bike });
+                    }
+                } else if (typeStr == QLatin1String("PARK")) { // this means "drive to parking space", not "park the car"...
+                    section.setMode(JourneySection::IndividualTransport);
+                    section.setIndividualTransport({ IndividualTransport::Car, IndividualTransport::Park });
+                } else if (typeStr == QLatin1String("CHKO")) { // ... while this means "park the car"
+                    section.setMode(JourneySection::Transfer); // ### we don't have any metter mode for this atm
+                } else if (typeStr == QLatin1String("KISS")) {
+                    section.setMode(JourneySection::RentedVehicle);
+                    RentalVehicle v;
+                    v.setType(RentalVehicle::Car);
+                    section.setRentalVehicle(v);
+                } else {
+                    qCWarning(Log) << "Unhandled section mode:" << typeStr;
+                }
             }
 
             sections.push_back(section);
@@ -714,3 +803,8 @@ QDateTime HafasMgateParser::parseDateTime(const QString &date, const QJsonValue
 
     return dt;
 }
+
+void HafasMgateParser::setPreferLineNumberProducts(std::vector<int> &&lineNumberProducts)
+{
+    m_lineNumberProducts = std::move(lineNumberProducts);
+}
