@@ -50,8 +50,7 @@ public:
     Platform departurePlatformLayout;
     Vehicle arrivalVehicleLayout;
     Platform arrivalPlatformLayout;
-
-    int estimatedDistance() const;
+    IndividualTransport individualTransport;
 };
 
 class JourneyPrivate : public QSharedData
@@ -79,27 +78,7 @@ KPUBLICTRANSPORT_MAKE_PROPERTY(JourneySection, Vehicle, departureVehicleLayout,
 KPUBLICTRANSPORT_MAKE_PROPERTY(JourneySection, Platform, departurePlatformLayout, setDeparturePlatformLayout)
 KPUBLICTRANSPORT_MAKE_PROPERTY(JourneySection, Vehicle, arrivalVehicleLayout, setArrivalVehicleLayout)
 KPUBLICTRANSPORT_MAKE_PROPERTY(JourneySection, Platform, arrivalPlatformLayout, setArrivalPlatformLayout)
-
-int JourneySectionPrivate::estimatedDistance() const
-{
-    assert(from.hasCoordinate() && to.hasCoordinate());
-    int d = 0;
-
-    float startLat = from.latitude();
-    float startLon = from.longitude();
-
-    for (const auto &stop : intermediateStops) {
-        if (!stop.stopPoint().hasCoordinate()) {
-            continue;
-        }
-        d += Location::distance(startLat, startLon, stop.stopPoint().latitude(), stop.stopPoint().longitude());
-        startLat = stop.stopPoint().latitude();
-        startLon = stop.stopPoint().longitude();
-    }
-
-    d += Location::distance(startLat, startLon, to.latitude(), to.longitude());
-    return std::max(d, path.distance());
-}
+KPUBLICTRANSPORT_MAKE_PROPERTY(JourneySection, KPublicTransport::IndividualTransport, individualTransport, setIndividualTransport)
 
 bool JourneySection::hasExpectedDepartureTime() const
 {
@@ -137,10 +116,25 @@ int JourneySection::distance() const
     if (d->mode == JourneySection::Waiting) {
         return 0;
     }
-    if (!d->from.hasCoordinate() || !d->to.hasCoordinate()) {
-        return d->distance;
+
+    int dist = 0;
+    if (d->from.hasCoordinate() && d->to.hasCoordinate()) {
+        float startLat = d->from.latitude();
+        float startLon = d->from.longitude();
+
+        for (const auto &stop : d->intermediateStops) {
+            if (!stop.stopPoint().hasCoordinate()) {
+                continue;
+            }
+            dist += Location::distance(startLat, startLon, stop.stopPoint().latitude(), stop.stopPoint().longitude());
+            startLat = stop.stopPoint().latitude();
+            startLon = stop.stopPoint().longitude();
+        }
+
+        dist += Location::distance(startLat, startLon, d->to.latitude(), d->to.longitude());
     }
-    return std::max(d->estimatedDistance(), d->distance);
+    dist = std::max(dist, d->path.distance());
+    return std::max(dist, d->distance);
 }
 
 void JourneySection::setDistance(int value)
@@ -352,12 +346,31 @@ QVariantList JourneySection::loadInformationVariant() const
     return l;
 }
 
+void JourneySection::applyMetaData(bool download)
+{
+    if (!from().hasCoordinate() || mode() != JourneySection::PublicTransport) {
+        return;
+    }
+    auto line = d->route.line();
+    line.applyMetaData(from(), download);
+    d->route.setLine(line);
+
+    // propagate to intermediate stops
+    for (auto &stop : d->intermediateStops) {
+        stop.setRoute(d->route);
+    }
+}
+
 bool JourneySection::isSame(const JourneySection &lhs, const JourneySection &rhs)
 {
     if (lhs.d->mode != rhs.d->mode) {
         return false;
     }
 
+    if (lhs.d->mode == JourneySection::IndividualTransport && lhs.d->individualTransport != rhs.d->individualTransport) {
+        return false;
+    }
+
     // we have N criteria to compare here, with 3 possible results:
     // - equal
     // - similar-ish, unknwon, or at least not conflicting
@@ -431,7 +444,7 @@ JourneySection JourneySection::merge(const JourneySection &lhs, const JourneySec
     res.d->loadInformation = LoadUtil::merge(lhs.d->loadInformation, rhs.d->loadInformation);
     res.d->rentalVehicle = RentalVehicleUtil::merge(lhs.d->rentalVehicle, rhs.d->rentalVehicle);
 
-    res.d->path = lhs.d->path.isEmpty() ? rhs.d->path : lhs.d->path;
+    res.d->path = lhs.d->path.sections().size() < rhs.d->path.sections().size() ? rhs.d->path : lhs.d->path;
 
     res.d->departureVehicleLayout = Vehicle::merge(lhs.d->departureVehicleLayout, rhs.d->departureVehicleLayout);
     res.d->departurePlatformLayout = Platform::merge(lhs.d->departurePlatformLayout, rhs.d->departurePlatformLayout);
@@ -490,6 +503,10 @@ QJsonObject JourneySection::toJson(const JourneySection &section)
         obj.insert(QLatin1String("arrivalPlatformLayout"), Platform::toJson(section.arrivalPlatformLayout()));
     }
 
+    if (section.mode() == JourneySection::IndividualTransport) {
+        obj.insert(QLatin1String("individualTransport"), IndividualTransport::toJson(section.individualTransport()));
+    }
+
     if (obj.size() <= 3) { // only the disruption and mode enums and distance, ie. this is an empty object
         return {};
     }
@@ -515,6 +532,8 @@ JourneySection JourneySection::fromJson(const QJsonObject &obj)
     section.setDeparturePlatformLayout(Platform::fromJson(obj.value(QLatin1String("departurePlatformLayout")).toObject()));
     section.setArrivalVehicleLayout(Vehicle::fromJson(obj.value(QLatin1String("arrivalVehicleLayout")).toObject()));
     section.setArrivalPlatformLayout(Platform::fromJson(obj.value(QLatin1String("arrivalPlatformLayout")).toObject()));
+    section.setIndividualTransport(IndividualTransport::fromJson(obj.value(QLatin1String("individualTransport")).toObject()));
+    section.applyMetaData(false);
     return section;
 }
 
@@ -564,6 +583,11 @@ bool Journey::hasExpectedDepartureTime() const
     return d->sections.empty() ? false : d->sections.front().hasExpectedDepartureTime();
 }
 
+QDateTime Journey::expectedDepartureTime() const
+{
+    return d->sections.empty() ? QDateTime() : d->sections.front().expectedDepartureTime();
+}
+
 int Journey::departureDelay() const
 {
     return d->sections.empty() ? 0 : d->sections.front().departureDelay();
@@ -582,6 +606,11 @@ bool Journey::hasExpectedArrivalTime() const
     return d->sections.empty() ? false : d->sections.back().hasExpectedArrivalTime();
 }
 
+QDateTime Journey::expectedArrivalTime() const
+{
+    return d->sections.empty() ? QDateTime() : d->sections.back().expectedArrivalTime();
+}
+
 int Journey::arrivalDelay() const
 {
     return d->sections.empty() ? 0 : d->sections.back().arrivalDelay();
@@ -606,6 +635,20 @@ Disruption::Effect Journey::disruptionEffect() const
     return effect;
 }
 
+void Journey::applyMetaData(bool download)
+{
+    for (auto &sec : d->sections) {
+        sec.applyMetaData(download);
+    }
+}
+
+static bool isTransportSection(JourneySection::Mode mode)
+{
+    return mode == JourneySection::PublicTransport
+        || mode == JourneySection::RentedVehicle
+        || mode == JourneySection::IndividualTransport;
+}
+
 bool Journey::isSame(const Journey &lhs, const Journey &rhs)
 {
     auto lIt = lhs.sections().begin();
@@ -613,11 +656,11 @@ bool Journey::isSame(const Journey &lhs, const Journey &rhs)
 
     while (lIt != lhs.sections().end() || rIt != rhs.sections().end()) {
         // ignore non-transport sections
-        if (lIt != lhs.sections().end() && (*lIt).mode() != JourneySection::PublicTransport) {
+        if (lIt != lhs.sections().end() && !isTransportSection((*lIt).mode())) {
             ++lIt;
             continue;
         }
-        if (rIt != rhs.sections().end() && (*rIt).mode() != JourneySection::PublicTransport) {
+        if (rIt != rhs.sections().end() && !isTransportSection((*rIt).mode())) {
             ++rIt;
             continue;
         }
@@ -680,11 +723,10 @@ QJsonArray Journey::toJson(const std::vector<Journey> &journeys)
     return Json::toJson(journeys);
 }
 
-Journey KPublicTransport::Journey::fromJson(const QJsonObject &obj)
+Journey Journey::fromJson(const QJsonObject &obj)
 {
     Journey j;
     j.setSections(JourneySection::fromJson(obj.value(QLatin1String("sections")).toArray()));
-    JourneyUtil::applyMetaData(j, false);
     return j;
 }
 
