@@ -10,6 +10,8 @@
 #include "../geo/geojson_p.h"
 #include "location.h"
 
+#include <QLineF>
+
 using namespace KPublicTransport;
 
 namespace KPublicTransport {
@@ -17,12 +19,16 @@ class PathSectionPrivate : public QSharedData {
 public:
     QPolygonF path;
     QString description;
+    int floorLevelChange = 0;
+    PathSection::Maneuver maneuver = PathSection::Move;
 };
 }
 
 KPUBLICTRANSPORT_MAKE_GADGET(PathSection)
 KPUBLICTRANSPORT_MAKE_PROPERTY(PathSection, QPolygonF, path, setPath)
 KPUBLICTRANSPORT_MAKE_PROPERTY(PathSection, QString, description, setDescription)
+KPUBLICTRANSPORT_MAKE_PROPERTY(PathSection, int, floorLevelChange, setFloorLevelChange)
+KPUBLICTRANSPORT_MAKE_PROPERTY(PathSection, PathSection::Maneuver, maneuver, setManeuver)
 
 int PathSection::distance() const
 {
@@ -38,12 +44,38 @@ int PathSection::distance() const
     return dist;
 }
 
+int PathSection::direction() const
+{
+    const auto p1 = startPoint();
+    const auto p2 = endPoint();
+    if (d->path.size() < 2 || p1 == p2) {
+        return -1;
+    }
+    return static_cast<int>(450 - QLineF(p1.x(), -p1.y(), p2.x(), -p2.y()).angle()) % 360;
+}
+
+QPointF PathSection::startPoint() const
+{
+    return d->path.empty() ? QPointF() : d->path.constFirst();
+}
+
+QPointF PathSection::endPoint() const
+{
+    return d->path.empty() ? QPointF() : d->path.constLast();
+}
+
 QJsonObject PathSection::toJson(const PathSection &section)
 {
     auto obj = Json::toJson(section);
     if (!section.path().empty()) {
         obj.insert(QLatin1String("path"), GeoJson::writeLineString(section.path()));
     }
+    if (section.maneuver() == PathSection::Move) {
+        obj.remove(QLatin1String("maneuver"));
+    }
+    if (section.floorLevelChange() == 0) {
+        obj.remove(QLatin1String("floorLevelChange"));
+    }
     return obj;
 }
 
@@ -101,6 +133,16 @@ int Path::distance() const
     return std::accumulate(d->sections.begin(), d->sections.end(), 0, [](int d, const auto &sec) { return d + sec.distance(); });
 }
 
+QPointF Path::startPoint() const
+{
+    return isEmpty() ? QPointF() : d->sections.front().startPoint();
+}
+
+QPointF Path::endPoint() const
+{
+    return isEmpty() ? QPointF() : d->sections.front().endPoint();
+}
+
 QJsonObject Path::toJson(const Path &path)
 {
     auto obj = Json::toJson(path);
